use crate::vulkan::{VkDevice, VkInstance, VkPhysicalDevice, VkSurface};
use ash::vk::QueueFlags;

pub struct VkRenderContext {
    instance: VkInstance,
    surface: VkSurface,
}

impl VkRenderContext {
    pub fn init(window: &crate::display::Window) -> anyhow::Result<Self> {
        let required_extensions = window
            .required_extensions()?;

        let instance = VkInstance::new(&required_extensions);
        let surface = instance.create_surface(&window)?;

        let mut physical_devices = instance.get_physical_devices();
        physical_devices.sort_by(|a, b| b.priority().cmp(&a.priority()));

        let (physical_device, queue_family_index, _) = VkPhysicalDevice::pick_physical_device_and_queue_by(
            &physical_devices,
            Some(QueueFlags::GRAPHICS),
            Some(&surface),
        ).expect("Unable to find physical device");

        let device = VkDevice::new_graphics_device(&instance, &physical_device, queue_family_index)
            .expect("Unable to create device");

        let present_queue = device.get_device_queue(0);

        let surface_format = surface.get_physical_device_surface_formats(physical_device)
            .unwrap_or_default()
            .first()
            .expect("Unable to get surface format");

        let surface_capabilities = surface.get_physical_device_surface_capabilities(physical_device)
            .expect("Unable to get surface capabilities");

        Ok(Self {
            instance,
            surface,
        })
    }
}